@@ -2,6 +2,7 @@ import os
 import numpy as np
 import os
 from SigProfilerMatrixGenerator.scripts import SigProfilerMatrixGeneratorFunc as matGen
+from SigProfilerHotSpots import hotspot
 import shutil
 from numpy import median
 import pickle
@@ -209,7 +210,8 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 					# 	print(samp, regions[samp][bisect.bisect_left(regions[samp], pos + chromLengths[genome][chrom])])
 					# 	print(chrom, chromLengths[genome][chrom], pos)
 					# 	continue
-					if pos - prev_pos < cutoff or (correction and ((regions[samp][bisect.bisect_left(regions[samp], pos + chromLengths[genome][chrom])] - (pos + chromLengths[genome][chrom]) < windowSize) and (pos - prev_pos) < imds_corrected[samp][regions[samp][bisect.bisect_left(regions[samp], pos + chromLengths[genome][chrom])]])):
+					if pos - prev_pos < cutoff or (correction and ((regions[samp][hotspot.catch([".",".",chrom, pos], regions[samp], chromLengths, genome, imds_corrected[samp])] - (pos + chromLengths[genome][chrom]) < windowSize) and (pos - prev_pos) < imds_corrected[samp][regions[samp][hotspot.catch([".",".",chrom, pos], regions[samp], chromLengths, genome, imds_corrected[samp])]])):
+					# if pos - prev_pos < cutoff or (correction and ((regions[samp][bisect.bisect_left(regions[samp], pos + chromLengths[genome][chrom])] - (pos + chromLengths[genome][chrom]) < windowSize) and (pos - prev_pos) < imds_corrected[samp][regions[samp][bisect.bisect_left(regions[samp], pos + chromLengths[genome][chrom])]])):
 						distances.append(pos-prev_pos)
 						mnv_length += 1
 						lines[i-1] = [str(count)] + lines[i-1]
@@ -297,8 +299,10 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 			print("HEADER", file=out7)
 			print("HEADER", file=out8)
 			for line in f:
-				line = line.strip().split()[1:]
+				# line = line.strip().split()[1:] 
+				line = line.strip().split()
 				if line != []:
+					line = line[1:-2] + [line[0]] + line[-2:]
 					lines.append(line)
 				else:
 					write_out = False
@@ -320,19 +324,27 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 							lines = []
 							continue
 						else:
-							# print(imdsData[lines[0][1]])
-							# print(chromLengths[genome][linesa[0][5]])
-							# print(regions[lines[0][1]][bisect.bisect_left(regions[lines[0][1]], (int(lines[0][7]) + chromLengths[genome][lines[0][5]]))])
-							# print((regions[lines[0][1]][bisect.bisect_left(regions[lines[0][1]], (int(lines[0][7]) + chromLengths[genome][lines[0][5]]))] - (int(lines[0][7]) + chromLengths[genome][lines[0][5]])))
-							# print(imds_corrected[lines[0][1]][regions[lines[0][1]][bisect.bisect_left(regions[lines[0][1]], int(lines[0][7]) + chromLengths[genome][lines[0][5]])]])
 							if correction:
-								distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]] or (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])
-								distancesFailed = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if (int(y[7])-int(x[7]) > imdsData[y[1]] and (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])
+								# distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]] or (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])
+								# distancesFailed = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if (int(y[7])-int(x[7]) > imdsData[y[1]] and (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])
+								# zeroDistances = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1])
+								distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]] or (regions[y[1]][hotspot.catch([".",".",y[5], y[7]], regions[y[1]], chromLengths, genome, imds_corrected[y[1]])] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][hotspot.catch([".",".",y[5], y[7]], regions[y[1]], chromLengths, genome, imds_corrected[y[1]])]]))])
+								distancesFailed = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if (int(y[7])-int(x[7]) > imdsData[y[1]] and (regions[y[1]][hotspot.catch([".",".",y[5], y[7]], regions[y[1]], chromLengths, genome, imds_corrected[y[1]])] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][hotspot.catch([".",".",y[5], y[7]], regions[y[1]], chromLengths, genome, imds_corrected[y[1]])]]))])
 								zeroDistances = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1])
+
 							else:
 								distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]])])
 								distancesFailed = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if (int(y[7])-int(x[7]) > imdsData[y[1]])])
 								zeroDistances = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1])
+							# if correction:
+							# 	distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]] or (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])
+							# 	distancesFailed = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if (int(y[7])-int(x[7]) > imdsData[y[1]] and (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])
+							# 	zeroDistances = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1])
+							# else:
+							# 	distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]])])
+							# 	distancesFailed = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if (int(y[7])-int(x[7]) > imdsData[y[1]])])
+							# 	zeroDistances = len([int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:]) if int(y[7])-int(x[7]) > 1])
+
 							vafs = len([abs(float(y[-1]) - float(x[-1])) for x,y in zip(lines, lines[1:]) if round(abs(float(y[-1]) - float(x[-1])),4) > vaf_cut])
 							unknownVafs = len([abs(float(y[-1]) - float(x[-1])) for x,y in zip(lines, lines[1:]) if abs(float(y[-1]) - float(x[-1])) < 0])
 							distancesActual = [int(y[7])-int(x[7]) for x,y in zip(lines, lines[1:])]
@@ -341,13 +353,24 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 							if unknownVafs == 0 and vafs == 0 and distancesFailed == 0:
 								# Class I or II
 								if distancesLine > 1:
-									writeClassII = True
-									if str(len(lines)) not in len_mnvs['II']:
-										len_mnvs['II'][str(len(lines))] = [1,distancesActual, vafsActual]
+									# Class IC or II
+									if len(lines) >= 4:
+										writeClassII = True
+										if str(len(lines)) not in len_mnvs['II']:
+											len_mnvs['II'][str(len(lines))] = [1,distancesActual, vafsActual]
+										else:
+											len_mnvs['II'][str(len(lines))][0] += 1
+											len_mnvs['II'][str(len(lines))][1] += distancesActual
+											len_mnvs['II'][str(len(lines))][2] += vafsActual
 									else:
-										len_mnvs['II'][str(len(lines))][0] += 1
-										len_mnvs['II'][str(len(lines))][1] += distancesActual
-										len_mnvs['II'][str(len(lines))][2] += vafsActual
+										writeClassI = True
+										writeClassIc = True
+										if str(len(lines)) not in len_mnvs['Ic']:
+											len_mnvs['Ic'][str(len(lines))] = [1,distancesActual, vafsActual]
+										else:
+											len_mnvs['Ic'][str(len(lines))][0] += 1
+											len_mnvs['Ic'][str(len(lines))][1] += distancesActual
+											len_mnvs['Ic'][str(len(lines))][2] += vafsActual										
 								else:
 									writeClassI = True
 									if str(len(lines)) not in len_mnvs['I']:
@@ -404,7 +427,7 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 									print(lines)
 
 								if writeClassIc:
-									# Writes Class Ic (extended MBSs) 
+									# Writes Class Ic (Omiklis) 
 									try:
 										for i in range(0, len(lines), 1):
 											lines[i][-1] = "ClassIC"
@@ -420,14 +443,11 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 									except:
 										print(lines)	
 								elif writeClassIb:
-									# print("yes")
 									# Writes Class 1b (MBSs)
 									try:
 										for i in range(0, len(lines), 1):
 											lines[i][-1] = "ClassIB"
-											# lines[i].append(category)
 											print("\t".join([x for x in lines[i]]), file=out7)
-										# print("yes", lines)
 									except:
 										print(lines)
 							elif writeClassIII:
@@ -456,7 +476,8 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 														lineRef = linesSubClass[i]																			
 											else:
 												if correction:
-													if abs(float(linesSubClass[i][-1]) - float(lineRef[-1])) < vaf_cut and (int(linesSubClass[i][7])-int(lineRef[7]) <= imdsData[lineRef[1]] or (regions[lineRef[1]][bisect.bisect_left(regions[lineRef[1]], (int(lineRef[7]) + chromLengths[genome][lineRef[5]]))] - (int(lineRef[7]) + chromLengths[genome][lineRef[5]]) < windowSize and int(lineRef[-2]) < imds_corrected[lineRef[1]][regions[lineRef[1]][bisect.bisect_left(regions[lineRef[1]], int(lineRef[7]) + chromLengths[genome][lineRef[5]])]])):
+													if abs(float(linesSubClass[i][-1]) - float(lineRef[-1])) < vaf_cut and (int(linesSubClass[i][7])-int(lineRef[7]) <= imdsData[lineRef[1]] or (regions[lineRef[1]][hotspot.catch([".",".",lineRef[5], lineRef[7]], regions[lineRef[1]], chromLengths, genome, imds_corrected[lineRef[1]])] - (int(lineRef[7]) + chromLengths[genome][lineRef[5]]) < windowSize and int(lineRef[-2]) < imds_corrected[lineRef[1]][regions[lineRef[1]][hotspot.catch([".",".",lineRef[5], lineRef[7]], regions[lineRef[1]], chromLengths, genome, imds_corrected[lineRef[1]])]])):
+													# if abs(float(linesSubClass[i][-1]) - float(lineRef[-1])) < vaf_cut and (int(linesSubClass[i][7])-int(lineRef[7]) <= imdsData[lineRef[1]] or (regions[lineRef[1]][bisect.bisect_left(regions[lineRef[1]], (int(lineRef[7]) + chromLengths[genome][lineRef[5]]))] - (int(lineRef[7]) + chromLengths[genome][lineRef[5]]) < windowSize and int(lineRef[-2]) < imds_corrected[lineRef[1]][regions[lineRef[1]][bisect.bisect_left(regions[lineRef[1]], int(lineRef[7]) + chromLengths[genome][lineRef[5]])]])):
 														saveNewEvent.append(linesSubClass[i])
 														lineRef = linesSubClass[i]	
 												else:
@@ -467,7 +488,8 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 
 									if len(saveNewEvent) > 1:
 										if correction:
-											distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(saveNewEvent, saveNewEvent[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]] or (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])
+											# distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(saveNewEvent, saveNewEvent[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]] or (regions[y[1]][bisect.bisect_left(regions[y[1]], (int(y[7]) + chromLengths[genome][y[5]]))] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][bisect.bisect_left(regions[y[1]], int(y[7]) + chromLengths[genome][y[5]])]]))])											
+											distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(saveNewEvent, saveNewEvent[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]] or (regions[y[1]][hotspot.catch([".",".",y[5], y[7]], regions[y[1]], chromLengths, genome, imds_corrected[y[1]])] - (int(y[7]) + chromLengths[genome][y[5]]) < windowSize and int(y[-2]) < imds_corrected[y[1]][regions[y[1]][hotspot.catch([".",".",y[5], y[7]], regions[y[1]], chromLengths, genome, imds_corrected[y[1]])]]))])
 										else:
 											distancesLine = len([int(y[7])-int(x[7]) for x,y in zip(saveNewEvent, saveNewEvent[1:]) if int(y[7])-int(x[7]) > 1 and (int(y[7])-int(x[7]) <= imdsData[y[1]])])
 										vafs = len([abs(float(y[-1]) - float(x[-1])) for x,y in zip(saveNewEvent, saveNewEvent[1:]) if round(abs(float(y[-1]) - float(x[-1])),4) > vaf_cut])
@@ -476,52 +498,56 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 										vafsActual = [abs(float(y[-1]) - float(x[-1])) for x,y in zip(saveNewEvent, saveNewEvent[1:])]
 										# Class I or II
 										if distancesLine > 1:
-											writeClassII = True
+											if len(lines) >=4:
+												writeClassII = True
+											else:
+												writeClassI = True
+												writeClassIc = True
 										else:
 											writeClassI = True
-											if str(len(saveNewEvent)) not in len_mnvs['I']:
-												len_mnvs['I'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
-											else:
-												len_mnvs['I'][str(len(saveNewEvent))][0] += 1
-												len_mnvs['I'][str(len(saveNewEvent))][1] += distancesActual
-												len_mnvs['I'][str(len(saveNewEvent))][2] += vafsActual
+											# if str(len(saveNewEvent)) not in len_mnvs['I']:
+											# 	len_mnvs['I'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
+											# else:
+											# 	len_mnvs['I'][str(len(saveNewEvent))][0] += 1
+											# 	len_mnvs['I'][str(len(saveNewEvent))][1] += distancesActual
+											# 	len_mnvs['I'][str(len(saveNewEvent))][2] += vafsActual
 											if distancesLine == 0:
 												if len(saveNewEvent) == 2:
 													writeClassIa = True
-													if str(len(saveNewEvent)) not in len_mnvs['Ia']:
-														len_mnvs['Ia'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
-													else:
-														len_mnvs['Ia'][str(len(saveNewEvent))][0] += 1
-														len_mnvs['Ia'][str(len(saveNewEvent))][1] += distancesActual
-														len_mnvs['Ia'][str(len(saveNewEvent))][2] += vafsActual
+													# if str(len(saveNewEvent)) not in len_mnvs['Ia']:
+													# 	len_mnvs['Ia'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
+													# else:
+													# 	len_mnvs['Ia'][str(len(saveNewEvent))][0] += 1
+													# 	len_mnvs['Ia'][str(len(saveNewEvent))][1] += distancesActual
+													# 	len_mnvs['Ia'][str(len(saveNewEvent))][2] += vafsActual
 												else:
 													writeClassIb = True
-													if str(len(saveNewEvent)) not in len_mnvs['Ib']:
-														len_mnvs['Ib'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
-													else:
-														len_mnvs['Ib'][str(len(saveNewEvent))][0] += 1
-														len_mnvs['Ib'][str(len(saveNewEvent))][1] += distancesActual
-														len_mnvs['Ib'][str(len(saveNewEvent))][2] += vafsActual
+													# if str(len(saveNewEvent)) not in len_mnvs['Ib']:
+													# 	len_mnvs['Ib'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
+													# else:
+													# 	len_mnvs['Ib'][str(len(saveNewEvent))][0] += 1
+													# 	len_mnvs['Ib'][str(len(saveNewEvent))][1] += distancesActual
+													# 	len_mnvs['Ib'][str(len(saveNewEvent))][2] += vafsActual
 											else:
 												writeClassIc = True
-												if str(len(saveNewEvent)) not in len_mnvs['Ic']:
-													len_mnvs['Ic'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
-												else:
-													len_mnvs['Ic'][str(len(saveNewEvent))][0] += 1
-													len_mnvs['Ic'][str(len(saveNewEvent))][1] += distancesActual
-													len_mnvs['Ic'][str(len(saveNewEvent))][2] += vafsActual
+												# if str(len(saveNewEvent)) not in len_mnvs['Ic']:
+												# 	len_mnvs['Ic'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
+												# else:
+												# 	len_mnvs['Ic'][str(len(saveNewEvent))][0] += 1
+												# 	len_mnvs['Ic'][str(len(saveNewEvent))][1] += distancesActual
+												# 	len_mnvs['Ic'][str(len(saveNewEvent))][2] += vafsActual
 									else:
 										writeClassIII = True
 										if unknownVafs > 0:
 											category = 'unknown'
 										else:
 											category = "vaf"
-										if str(len(saveNewEvent)) not in len_mnvs['III']:
-											len_mnvs['III'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
-										else:
-											len_mnvs['III'][str(len(saveNewEvent))][0] += 1
-											len_mnvs['III'][str(len(saveNewEvent))][1] += distancesActual
-											len_mnvs['III'][str(len(saveNewEvent))][2] += vafsActual
+										# if str(len(saveNewEvent)) not in len_mnvs['III']:
+										# 	len_mnvs['III'][str(len(saveNewEvent))] = [1,distancesActual, vafsActual]
+										# else:
+										# 	len_mnvs['III'][str(len(saveNewEvent))][0] += 1
+										# 	len_mnvs['III'][str(len(saveNewEvent))][1] += distancesActual
+										# 	len_mnvs['III'][str(len(saveNewEvent))][2] += vafsActual
 									for line in saveNewEvent:
 										linesSubClass.remove(line)
 
@@ -596,10 +622,6 @@ def findClustersOfClusters (project, chrom_based, project_parent_path, windowSiz
 
 		with open(project_path + "clusteredStats" + path_suffix + ".pickle", "wb") as f:
 			pickle.dump(len_mnvs, f)
-		# for classes in len_mnvs:
-		# 	# print(classes)
-		# 	for leng in len_mnvs[classes]:
-		# 		print("\t",leng, len_mnvs[classes][leng][0], median(len_mnvs[classes][leng][1]))
 
 
 	try:
